package org.deken.game.component;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import org.deken.game.animation.Animation;
import org.deken.game.input.InputAction;
import org.deken.game.input.InputListener;
import org.deken.game.input.InputMouseAction;
import org.deken.game.map.GameLocation;
import org.deken.game.sprites.Sprite;
import org.deken.game.sprites.SpriteSize;

/**
 * <p>Title: GameButton</p>
 *
 * <p>Description: GameButton component, similar to the Swing component, but is
 * modeled after the sprite. It's location is always from 0,0, and is displayed
 * statically.</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * <p>@author Gary Deken</p>
 *
 * @version 1.0
 */
public class GButton extends GTextContainer implements InputListener {
   /**
    * Button Effects.
    */
   public static final int ANIMATION_DEFAULT = 0;
   public static final int ANIMATION_ROLLOVER = 1;
   public static final int ANIMATION_PRESSED = 2;
   public static final int ANIMATION_DIABLED = 3;
   protected Animation[] animations = new Animation[4];
   private InputAction keyAction;
   private Color fontHighlightColor;
   // Button animations
   private int animationIndex = 0;
   private InputAction action;
   private GComponent.Alignment alignment = Alignment.LEFT;
   private String keyAssociated = "";
   private boolean selected = false;

   /**
    * GameButton
    *
    * @param inputMouseAction
    */
   public GButton(InputMouseAction inputMouseAction) {
      this("", inputMouseAction);
   }

   /**
    * GameButton
    *
    * @param text String
    * @param inputMouseAction
    */
   public GButton(String text, InputMouseAction inputMouseAction) {
      this(text, new Font(Font.SANS_SERIF, Font.PLAIN, 10), Color.black, Color.GRAY, inputMouseAction);
   }

   /**
    * GameButton
    *
    * @param animation Animation
    * @param inputMouseAction
    */
   public GButton(Animation animation, InputMouseAction inputMouseAction) {
      super(inputMouseAction);
      this.animations[0] = animation;
      if (animation != null) {
         size = new SpriteSize(animations[0].getWidth(), animations[0].getHeight());
      }
   }

   /**
    *
    * @param text
    * @param font
    * @param color
    * @param inputMouseAction
    */
   public GButton(String text, Font font, Color fontColor, Color fontHightlightColor, InputMouseAction inputMouseAction) {
      this(new GText(text, font, fontColor), fontHightlightColor, inputMouseAction);
   }

   public GButton(GText gameText, Color fontHightlightColor, InputMouseAction inputMouseAction) {
      super(inputMouseAction);
      this.gameText = gameText;
      this.fontHighlightColor = fontHightlightColor;
      invalid = true;
   }

   /**
    * Add a key input action, that will trigger this button.
    *
    * @param keyAction
    */
   public void addKeyAction(InputAction keyAction) {
      this.keyAction = keyAction;
   }

   /**
    * Listeners for when this button is pressed.
    *
    * @param listener
    * @return if added.
    */
   public boolean addListener(InputListener listener) {
      if (action == null) {
         action = new InputAction(getName(), InputAction.Behavior.NORMAL);
      }
      return action.add(listener);

   }

   /**
    * Receive Key actions.
    *
    * @param inputAction
    */
   public void notifyListener(InputAction inputAction) {
      if (inputAction.isActive() && isEnabled()) {
      }
   }

   /**
    * Receive Mouse actions. The mouse hover action does not set the selected
    * boolean. The selected boolean is when this button is selected by another
    * means.
    *
    * @param inputMouseAction
    */
   public void notifyListener(InputMouseAction inputMouseAction) {
      if (inputMouseAction.isMovement()) {
         if (getBounds().contains(inputMouseAction.getXPosition(), inputMouseAction.getYPosition())) {
            if (animations[ANIMATION_ROLLOVER] != null || gameText != null) {
               animationIndex = ANIMATION_ROLLOVER;
            }
         }
         else {
            animationIndex = ANIMATION_DEFAULT;
         }
      }
      else if (getBounds().contains(inputMouseAction.getXPosition(), inputMouseAction.getYPosition())) {
         // If pressed
         if (inputMouseAction.isActive()) {
            if (animations[ANIMATION_PRESSED] != null) {
               animationIndex = ANIMATION_PRESSED;
            }
            if (action != null) {
               action.activate();
            }
         }
         // if released.
         else if (action != null) {
            action.deactivate();
         }
      }
   }

   /**
    * @param graphics
    * @param xOffset
    * @param yOffset
    */
   @Override
   public void draw(Graphics2D graphics, int xOffset, int yOffset) {
      if (invalid) {
         validate(graphics);
      }
      if (animations[0] != null) {
         if (parent != null) {
            animations[animationIndex].draw(graphics, xOffset + (int) location.x, yOffset + (int) location.y);
         }
         else {
            animations[animationIndex].draw(graphics, (int) location.x, (int) location.y);
         }
      }
      if (gameText != null) {
         int x = (int) location.x;
         int y = (int) location.y;
         if (parent != null) {
            x += xOffset;
            y += yOffset;
         }
         if (gameText.getBackgroundColor() != null) {
            graphics.setColor(gameText.getBackgroundColor());
            graphics.fillRect(x, y, gameText.getSize().getWidth(), gameText.getSize().getHeight());
         }
         if (animationIndex == ANIMATION_ROLLOVER) {
            graphics.setColor(fontHighlightColor);
         }
         else {
            graphics.setColor(gameText.getFontColor());
         }
         graphics.setFont(gameText.getFont());
         graphics.drawString(gameText.getText(), x, y + gameText.getTextOffset());
      }
   }

   /**
    * update() <p>Update the button animation.</p>
    *
    * @param elapseTime long
    */
   public void update(long elapseTime) {
      if (gameText == null) {
         animations[animationIndex].update(elapseTime);
      }
   }

   /**
    * @param graphics
    */
   public void validate(Graphics2D graphics) {
      if (gameText != null) {
         gameText.validate(graphics);
         // get size for string and font.
         size = gameText.getSize();
      }
      invalid = false;
      positionHorizontalAlignment();
      positionVerticalAlignment();
   }

   /**
    *
    * @return InputAction
    */
   public InputAction getAction() {
      if (action == null) {
         action = new InputAction(getName(), InputAction.Behavior.NORMAL);
      }
      return action;
   }

   /**
    * getAlignment
    *
    * @return Alignment
    */
   public Alignment getAlignment() {
      return alignment;
   }

   public Animation getAnimation() {
      return animations[animationIndex];
   }

   /**
    * getImage<p>Returns the current image for the button.</p>
    *
    * @return Image
    */
   public Image getImage() {
      return animations[animationIndex].getImage();
   }

   public String getKeyAssociated() {
      return keyAssociated;
   }

   public Color getFontHighlightColor() {
      return fontHighlightColor;
   }

   public GText getGameText() {
      return gameText;
   }

   public boolean isSelected() {
      return selected;
   }

   /**
    * setEnabled
    *
    * <p>Calls the super method, and set the current animation to the enabled or
    * disabled.</p>
    *
    * @param b boolean
    */
   @Override
   public void setEnabled(boolean b) {
      super.setEnabled(b);
      if (b) {
         animationIndex = ANIMATION_DEFAULT;
      }
      else {
         animationIndex = ANIMATION_DIABLED;
      }
   }

   public void setGameText(GText gameText) {
      this.gameText = gameText;
      invalid = true;
   }

   /**
    * Makes this button selected.
    */
   public void setSelected() {
      if (animations[ANIMATION_ROLLOVER] != null) {
         animationIndex = ANIMATION_ROLLOVER;
      }
      selected = true;
   }

   /**
    * setAlignment
    *
    * @param alignment Alignment
    */
   public void setAlignment(Alignment alignment) {
      this.alignment = alignment;
   }

   public void setKeyAssociated(String keyAssociated) {
      this.keyAssociated = keyAssociated;
   }

   /**
    * Sets the animation to one of the four button effects: The default,
    * rollover, pressed and disabled.
    *
    * @param animation
    * @param buttonEffect
    */
   public void setAnimation(Animation animation, int buttonEffect) {
      animations[buttonEffect] = animation;
   }

   public void setFontHighlightColor(Color fontHighlightColor) {
      this.fontHighlightColor = fontHighlightColor;
   }

   /**
    *
    * @return GameButton
    */
   @Override
   public GButton copy() {
      GButton that = new GButton(mouseAction[0]);
      that.size = this.size.copy();
      that.location = new GameLocation(this.location.x, this.location.y);
      if (this.animations[0] != null) {
         that.animations[0] = this.animations[0].copy();
      }
      if (this.animations[1] != null) {
         that.animations[1] = this.animations[1].copy();
      }
      if (this.animations[2] != null) {
         that.animations[2] = this.animations[2].copy();
      }
      if (this.animations[3] != null) {
         that.animations[3] = this.animations[3].copy();
      }
      if (this.mouseAction[1] != null) {
         that.mouseAction[1] = this.mouseAction[1];
      }
      that.gameText = this.gameText.copy();
      that.fontHighlightColor = this.fontHighlightColor;
      that.invalid = true;

      return that;
   }

   @Override
   protected void spriteCollison(Sprite collideSprite) {
      // Not used.
   }
}
