package org.deken.game.component;

import java.awt.Graphics2D;
import org.deken.game.input.InputMouseAction;
import org.deken.game.input.InputMouseListener;
import org.deken.game.map.GameLocation;
import org.deken.game.sprites.Sprite;
import org.deken.game.sprites.SpriteSize;

/**
 * <p>Title: Game Component</p>
 *
 * <p>Description: This abstract object is an element that is used for user
 * interaction. Game components can be any type of element that the user will
 * directly interact with, such as a button, or window.</p>
 *
 * <p>Copyright: Copyright (c) 2006</p>
 *
 * <p>@author Gary Deken</p>
 *
 * @version 1.0
 */
public abstract class GComponent extends Sprite implements InputMouseListener {
   /*
    The game component has a location that is reletive to its parent. if there
    is no parent, the location is reletive to the top of the viewing screen. The
    game component has a heigth and width. It can be seen or not seen. It can be
    enabled or disabled.
    A game component can tell if it has been clicked by the mouse. It can tell if
    it has focus or not.
    */
   public final static int MOUSE_PRESS = 0;
   public final static int MOUSE_MOVE = 1;
   protected InputMouseAction[] mouseAction = new InputMouseAction[2];
   protected GContainer parent;
   protected int xOffSet = 0;
   protected int yOffSet = 0;
   protected boolean invalid = false;
   protected Alignment horizontalAlignment;
   protected Alignment verticalAlignment;
   protected int screenWidth = 0;
   protected int screenHeight = 0;
   private boolean enabled;

   public GComponent() {
      enabled = true;
      size = new SpriteSize();
      location = new GameLocation();
   }

   /**
    * GameComponent
    *
    * @param inputMouseAction
    */
   public GComponent(InputMouseAction inputMouseAction) {
      this();
      mouseAction[MOUSE_PRESS] = inputMouseAction;
      inputMouseAction.add(this);
   }

   /**
    * addInputAction
    *
    * @param inputMouseAction InputMouseAction
    * @param action int
    */
   public void addInputMouseAction(InputMouseAction inputMouseAction, int action) {
      inputMouseAction.add(this);
      mouseAction[action] = inputMouseAction;
   }

   /**
    * @return GameComponent
    */
   public abstract GComponent copy();

   /**
    * Draws the sprite with the current offset.
    *
    * @param graphics
    * @param xOffset
    * @param yOffset
    */
   public abstract void draw(Graphics2D graphics, int xOffset, int yOffset);

   /**
    *
    * @return Alignment
    */
   public Alignment getHorizontalAlignment() {
      return horizontalAlignment;
   }

   /**
    *
    * @return GameContainer
    */
   public GContainer getParent() {
      return parent;
   }

   /**
    *
    * @return Alignment
    */
   public Alignment getVerticalAlignment() {
      return verticalAlignment;
   }

   /**
    * getXLocation
    *
    * @return double
    */
   @Override
   public double getXLocation() {
      if (parent != null) {
         return parent.getLocation().getX() + location.getX();
      }
      return location.getX();
   }

   /**
    * getYLocation
    *
    * @return double
    */
   @Override
   public double getYLocation() {
      if (parent != null) {
         return parent.getLocation().getY() + location.getY();
      }
      return location.getY();
   }

   /**
    *
    * @return boolean
    */
   public boolean isEnabled() {
      return enabled;
   }

   /**
    *
    * @param enabled
    */
   public void setEnabled(boolean enabled) {
      this.enabled = enabled;
   }

   /**
    * Makes this GameComponent invalid, so that it will be re-validated.
    */
   public void setInvalid() {
      invalid = false;
   }

   /**
    *
    * @param parent
    */
   public void setParent(GContainer parent) {
      this.parent = parent;
   }

   /**
    * @param horizontalAlignment
    * @param screenWidth
    */
   public void setHorizontalAlignment(Alignment horizontalAlignment, int screenWidth) {
      this.horizontalAlignment = horizontalAlignment;
      this.screenWidth = screenWidth;
      invalid = true;
   }

   /**
    *
    * @param verticalAlignment
    * @param screenHeight
    */
   public void setVerticalAlignment(Alignment verticalAlignment, int screenHeight) {
      this.verticalAlignment = verticalAlignment;
      this.screenHeight = screenHeight;
      invalid = true;
   }

   /**
    * update
    *
    * <p>This method takes the elapse time, and calculates the new current frame
    * of the animation.</p>
    *
    * @param elapseTime long
    */
   public abstract void update(long elapseTime);

   /**
    *
    * @param graphics
    */
   public abstract void validate(Graphics2D graphics);

   /**
    * Calculates the vertical position based on the parent or screen height, and
    * the Alignment set.
    */
   protected void positionVerticalAlignment() {
      int height = screenHeight;
      if (parent != null) {
         height = parent.getHeight();
      }
      if (verticalAlignment != null) {
         switch (verticalAlignment) {
            case TOP:
               location.y = 0;
               break;
            case CENTER:
               location.y = (height / 2) - (size.getHeight() / 2);
               break;
            case BOTTOM:
               location.y = height - size.getHeight();
         }
      }
   }

   /**
    * Calculates the horizontal position based on the parent or screen width,
    * and the Alignment set.
    */
   protected void positionHorizontalAlignment() {
      int width = screenWidth;
      if (parent != null) {
         width = parent.getWidth();
      }
      if (horizontalAlignment != null) {
         switch (horizontalAlignment) {
            case LEFT:
               location.x = 0;
               break;
            case CENTER:
               location.x = (width / 2) - (size.getWidth() / 2);
               break;
            case RIGHT:
               location.x = width - size.getWidth();
         }
      }
   }

   public enum Alignment {
      RIGHT,
      LEFT,
      CENTER,
      TOP,
      BOTTOM;
   }

}
