/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.deken.game.component;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;
import org.deken.game.input.InputMouseAction;
import org.deken.game.map.GameLocation;
import org.deken.game.sprites.Sprite;
import org.deken.game.sprites.SpriteSize;

/**
 * <p>Title: GScrollingContainer</p>
 *
 * <p>Description: This container scrolls the content in a given direction.</p>
 *
 * Copyright: Copyright (c) Mar 23, 2013
 *
 * @author Gary Deken
 * @version 1.0
 */
public class GScrollingParagraph extends GComponent {
   private static final String NEW_LINE = "\r";
   private String text;
   private Font font;
   private Color fontColor = Color.WHITE;
   private Color backgroundColor;
   private int lineSpacing = 0;
   private int totalLineHeight;
   private int textYOffset;
   protected List<String> textList = new ArrayList<String>();
   private boolean indentNewLines = false;
   protected String indentString = "";
   private double speed;
   protected int startIndex = 0;
   protected double startSize = 0.0;
   protected double yTextOffset = 0;
   private boolean firstStart = false;
   private int maxDescent = 0;

   public GScrollingParagraph(String text, int width, int height, double speed) {
      super();
      this.text = text;
      size = new SpriteSize(width, height);
      this.speed = speed;
      this.font = new Font(Font.SANS_SERIF, Font.PLAIN, 10);
      invalid = true;
   }

   public GScrollingParagraph(String text, int width, int height, double speed, Font font, Color fontColor) {
      super();
      this.text = text;
      size = new SpriteSize(width, height);
      this.speed = speed;
      this.font = font;
      this.fontColor = fontColor;
      invalid = true;
   }

   /**
    * Adds a Blank line, with no text.
    */
   public void addBlankLine() {
      text += " " + NEW_LINE;
      invalid = true;
   }

   /**
    * Adds text as a new section. A carriage return is added before the new
    * text.
    *
    * @param textOnNewLine
    */
   public void addTextToSection(String textOnNewLine) {
      this.text += " " + NEW_LINE + textOnNewLine;
      invalid = true;
   }

   /**
    *
    * @return a copy of this GScrollingParagraph
    */
   public GScrollingParagraph copy() {
      GScrollingParagraph that = new GScrollingParagraph(text, size.getWidth(), size.getHeight(), speed);

      that.size = this.size.copy();
      that.font = this.font;
      that.fontColor = this.fontColor;
      that.backgroundColor = this.backgroundColor;
      that.location = new GameLocation(this.location.x, this.location.y);
      that.invalid = true;

      return that;
   }

   /**
    *
    * @param graphics
    * @param xOffset
    * @param yOffset
    */
   public void draw(Graphics2D graphics, int xOffset, int yOffset) {
      if (invalid) {
         validate(graphics);
      }
      int xLocation, yLocation;
      if (parent != null) {
         xLocation = xOffset + (int) location.x;
         yLocation = yOffset + (int) location.y;
      }
      else {
         xLocation = (int) location.x;
         yLocation = (int) location.y;
      }
      if (backgroundColor != null) {
         graphics.setColor(backgroundColor);
         graphics.fillRect(xLocation, (int) location.y, size.getWidth(), size.getHeight());
      }
      // add total height so we can scroll up.
      yLocation += (totalLineHeight * 2);
      graphics.setColor(fontColor);
      graphics.setFont(font);
      int index = startIndex;
      int currentHeight = 0;
      int bottomEdge = size.getHeight() - (totalLineHeight * 2) - maxDescent + (int) startSize;
      while (currentHeight < bottomEdge) {
         if (index >= textList.size()) {
            index = 0;
         }
         graphics.drawString(textList.get(index), xLocation, yLocation + currentHeight - (int) startSize);
         currentHeight += totalLineHeight;
         index++;
      }
   }

   public void notifyListener(InputMouseAction inputMouseAction) {
      throw new UnsupportedOperationException("Not supported yet.");
   }

   public void setBackgroundColor(Color backgroundColor) {
      this.backgroundColor = backgroundColor;
   }

   /**
    * Sets if new sections should be indented, and by how many spaces. This only
    * needs to be set once per GameParagraph
    *
    * @param indent
    * @param spaces
    */
   public void setIndentNewSection(boolean indent, int spaces) {
      this.indentNewLines = indent;
      if (indent) {
         for (int i = 0; i < spaces; i++) {
            this.indentString += " ";
         }
      }
      else {
         this.indentString = "";
      }
   }

   /**
    * Sets the number of pixels to put between lines of text. The default is 0.
    *
    * @param lineSpacing
    */
   public void setLineSpacing(int lineSpacing) {
      this.lineSpacing = lineSpacing;
      invalid = true;
   }

   /**
    * Set the text value to be display in the paragraph. This over-rides any
    * current text.
    *
    * @param text
    */
   public void setText(String text) {
      this.text = text;
      invalid = true;
   }

   /**
    *
    * @param elapseTime
    */
   public void update(long elapseTime) {
      if (firstStart) {
         // update the amount of movement during this sequence of time.
         yTextOffset = speed * elapseTime;
         startSize += yTextOffset;
         if (startSize > (totalLineHeight)) {
            startSize -= (totalLineHeight);
            startIndex++;
            if (startIndex >= textList.size()) {
               startIndex = 0;
            }
         }
      }
      else {
         firstStart = true;
         yTextOffset = 0;
      }
   }

   @Override
   public void validate(Graphics2D graphics) {
      textList.clear();
      int width = 0;
      FontMetrics fontMetrics = graphics.getFontMetrics(this.font);

      maxDescent = fontMetrics.getMaxDescent();
      int lineHeight = fontMetrics.getMaxAscent() + maxDescent;
      totalLineHeight = lineHeight + lineSpacing;
      textList.add(" ");

      StringBuilder strBld = new StringBuilder();
      StringTokenizer tokenizer = new StringTokenizer(text, " ");

      while (tokenizer.hasMoreTokens()) {
         String str = tokenizer.nextToken() + " ";
         boolean newLine = str.startsWith(NEW_LINE);
         if (newLine) {
            str = str.substring(1);
            if (indentNewLines) {
               str = indentString + str;
            }
         }
         int textWidth = fontMetrics.stringWidth(str);

         if (width + textWidth > size.getWidth() || newLine) {
            textList.add(strBld.toString());
            strBld = new StringBuilder();
            width = 0;
         }
         width += textWidth;
         strBld.append(str);
      }

      // set Y location so the text top is at original Y Location.
      // Add another line to allow us to scroll up one row.
      textYOffset = totalLineHeight;
      invalid = false;
   }

   protected void spriteCollison(Sprite collideSprite) {
      // Not used.
   }
}
